@@ -23,17 +23,20 @@
 
 #include "widgettextcaptureconfig.h"
 #include "ui_widgettextcaptureconfig.h"
+#include <klocalizedstring.h>
+#include <kconfiggroup.h>
+#include <ksharedconfig.h>
+#include <kconfig.h>
 
 WidgetTextCaptureConfig::WidgetTextCaptureConfig(QWidget* parent)
- : KDialog(parent)
+ : QDialog(parent)
  , ui(new Ui_WidgetTextCapture)
 {
     setAttribute(Qt::WA_DeleteOnClose, true);
-    ui->setupUi(mainWidget());
-    setCaption(i18nc("@title","Widget Text Capture"));
-    setButtons(KDialog::Ok | KDialog::Cancel);
+    ui->setupUi(this);
+    setWindowTitle(i18nc("@title","Widget Text Capture"));
 
-    KConfigGroup cg(KGlobal::config(), "Development");
+    KConfigGroup cg(KSharedConfig::openConfig(), "Development");
     bool copyWidgetText = cg.readEntry("CopyWidgetText", false);
     QString copyWidgetTextCommand = cg.readEntry("CopyWidgetTextCommand", QString());
     ui->none->setChecked(!copyWidgetText);
